﻿/*
 * Date: 05-Sep-2010
 * Author: Bhavin Panchal
 * Contact Email: bhavin_panchal777@hotmail.com
 * Application name: CrossCheckDBML
 * Purpose: To check the stored proc and functions in DBML file do exist on given database
 * Icons credit to: http://www.diveandgo.com
 */

using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Windows.Forms;

namespace CrossCheckDBML
{
    public partial class frmCrossCheckDBMLMain : Form
    {
        private DataSet ds = new DataSet();

        private const string cnst_xmltag_function = "Function";
        private const string cnst_xmlattr_Method = "Method";

        private const string cnst_DBAddress = "DBAddress";
        private const string cnst_DBUserName = "DBUserName";
        private const string cnst_DBName = "DBName";
        private const string cnst_CheckSP = "CheckSP";
        private const string cnst_CheckScalerFunction = "CheckScalerFunction";
        private const string cnst_DoRemember = "DoRemember";

        public frmCrossCheckDBMLMain()
        {
            InitializeComponent();

            Load += new EventHandler(frmCrossCheckDBMLMain_Load);
            KeyUp += new KeyEventHandler(frmCrossCheckDBMLMain_KeyUp);
        }

        void frmCrossCheckDBMLMain_KeyUp(object sender, KeyEventArgs e)
        {
            if (e != null)
            {
                if (e.KeyCode == Keys.Escape)
                {
                    Close();
                }
            }
        }

        /// <summary>
        /// Occurs when the application is loaded
        /// try to read the application settings and if available then set the Database ip address, name, username and options ticked during previous run
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void frmCrossCheckDBMLMain_Load(object sender, EventArgs e)
        {
            try
            {
                ReadAppSettings();
            }
            catch (Exception)
            {
                // bypass the exception
            }
        }

        /// <summary>
        /// Occurs when the "Open DBML" button is clicked
        /// When appropriate DBML file is selected, it opens and reads the DBML file 
        /// and loads the "Stored Proc" / "Functions" into the grid view
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpenFileDialog_Click(object sender, EventArgs e)
        {
            if (ofd1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    txtDBMLFile.Text = ofd1.FileName;
                    ds.ReadXml(ofd1.OpenFile());
                    dataGridView1.DataSource = ds.Tables[cnst_xmltag_function];
                    dataGridView1.Update();
                }
                catch (Exception)
                {
                    MessageBox.Show("Oops, error occurred while reading and loading dbml file");
                }
            }
        }

        /// <summary>
        /// Create the AppSettings section, and stores the value
        /// </summary>
        private void CreateAppSettings()
        {
            // Get the application configuration file.
            System.Configuration.Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);

            string sectionName = "appSettings";

            // Add an entry to appSettings.
            int appStgCnt = ConfigurationManager.AppSettings.Count;

            // before adding key and its value, clear out any existing value from it
            config.AppSettings.Settings.Clear();

            // add key and value
            config.AppSettings.Settings.Add(cnst_DBAddress, txtDBAddress.Text);
            config.AppSettings.Settings.Add(cnst_DBUserName, txtDBUserName.Text);
            config.AppSettings.Settings.Add(cnst_DBName, txtDBName.Text);
            config.AppSettings.Settings.Add(cnst_CheckSP, cbStoredProc.Checked.ToString());
            config.AppSettings.Settings.Add(cnst_CheckScalerFunction, cbScalerFunction.Checked.ToString());
            config.AppSettings.Settings.Add(cnst_DoRemember, cbRemember.Checked.ToString());

            // Save the configuration file.
            config.Save(ConfigurationSaveMode.Full);

            config.SaveAs(config.FilePath.Replace(".vshost", string.Empty));

            // Force a reload of the changed section. This 
            // makes the new values available for reading.
            ConfigurationManager.RefreshSection(sectionName);
        }


        /// <summary>
        /// Get the AppSettings and sets the value to UI controls
        /// </summary>
        public void ReadAppSettings()
        {
            try
            {
                Configuration config = ConfigurationManager.OpenExeConfiguration("CrossCheckDBML.exe");

                // Get the AppSettings section.
                AppSettingsSection appSettingsSection = config.AppSettings;

                KeyValueConfigurationCollection appSettings = config.AppSettings.Settings;
                
                txtDBAddress.Text = appSettings[cnst_DBAddress].Value;
                txtDBUserName.Text = appSettings[cnst_DBUserName].Value;
                txtDBName.Text = appSettings[cnst_DBName].Value;
                cbStoredProc.Checked = bool.Parse(appSettings[cnst_CheckSP].Value);
                cbScalerFunction.Checked = bool.Parse(appSettings[cnst_CheckScalerFunction].Value);
                cbRemember.Checked = bool.Parse(appSettings[cnst_DoRemember].Value);
            }
            catch (ConfigurationErrorsException)
            {
                // bypass the error
            }
        }

        /// <summary>
        /// Clears app settings
        /// </summary>
        public void ClearAppSettings()
        {
            // Get the application configuration file.
            System.Configuration.Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);

            // Add an entry to appSettings.
            int appStgCnt = ConfigurationManager.AppSettings.Count;

            config.AppSettings.Settings.Clear();

            // Save the configuration file.
            config.Save(ConfigurationSaveMode.Full);
            config.SaveAs(config.FilePath.Replace(".vshost", string.Empty));
        }

        private void btnConnectAndAnalyse_Click(object sender, EventArgs e)
        {
            try
            {
                // store the app settings if "Remember" was ticked, else clear the app settings
                if (cbRemember.Checked)
                {
                    CreateAppSettings();
                }
                else
                {
                    ClearAppSettings();
                }

                SqlConnection conn = new SqlConnection("Data Source=" + txtDBAddress.Text + ";Initial Catalog=" + txtDBName.Text + ";Persist Security Info=True;User ID=" + txtDBUserName.Text + ";Password=" + txtDBPassword.Text);
                conn.Open();

                int rowCount = dataGridView1.Rows.Count - 1;
                int rowCountTotalStoredProcAndFunctions = rowCount;
                int rowCountOfValidStoredProcAndFunctions = 0;

                SqlCommand command = conn.CreateCommand();

                bool isOnlyShowInvalid = cbOnlyShowInvalid.Checked;
                string strType = string.Empty;

                if (cbStoredProc.Checked) strType = "type = 'P'";

                if (strType.Length > 0)
                {
                    strType += "OR type = 'FN'";
                }
                else if (cbScalerFunction.Checked)
                {
                    strType = "type = 'FN'";
                }

                if (strType.Length > 0)
                {
                    strType = "(" + strType + ") AND ";
                }

                txtOutput.Text = "Start analysis for database " + txtDBName.Text + Environment.NewLine + Environment.NewLine;

                while (rowCount > 0)
                {
                    // read the method name from the grid view
                    string storedProcName = dataGridView1.Rows[rowCount - 1].Cells[cnst_xmlattr_Method].Value.ToString();

                    // build the command text and pass it to sql command
                    command.CommandText = "SELECT * FROM sysobjects WHERE " + strType + " name = '" + storedProcName + "'";

                    // execute the command that returns a SqlDataReader
                    SqlDataReader reader = command.ExecuteReader();

                    // default isExist false
                    bool isExist = false;

                    // display the results
                    while (reader.Read())
                    {
                        isExist = true;
                        rowCountOfValidStoredProcAndFunctions++;
                    }

                    if (isOnlyShowInvalid)
                    {
                        if (!isExist)
                            txtOutput.Text += "stored proc " + storedProcName + (isExist ? " Exist" : " do not exist") + Environment.NewLine;
                    }
                    else
                    {
                        txtOutput.Text += "stored proc " + storedProcName + (isExist ? " Exist" : " do not exist") + Environment.NewLine;
                    }

                    // close the connection
                    reader.Close();
                    rowCount--;
                    Application.DoEvents();
                }

                txtOutput.Text += "out of " + rowCountTotalStoredProcAndFunctions + " stored proc in dbml, there are " + rowCountOfValidStoredProcAndFunctions + " only in database";
                txtOutput.Text += Environment.NewLine + Environment.NewLine + "Analysis ends";
                conn.Close();
            }
            catch (Exception)
            {
                MessageBox.Show("Oops, error occurred while cross checking dbml");
            }
        }

        private void btnClearGrid_Click(object sender, EventArgs e)
        {
            dataGridView1.DataSource = null;
            dataGridView1.Update();
        }

        private void cbRemember_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                // store the app settings if "Remember" was ticked, else clear the app settings
                if (cbRemember.Checked)
                {
                    CreateAppSettings();
                }
                else
                {
                    ClearAppSettings();
                }
            }
            catch (Exception)
            {
                // by pass the exception
            }
        }
    }
}
